// This program shows how to walk through a directory.  It walks
// through the current directory. For each entry, it prints the name
// and size.

// Copyright David M. Hayden 2010

#include <hpgcc49.h>
#include "hpobjects.h"

void pause()
{
    printf("Press ON to continue\n");
    while (!keyb_isON()) ;	/* wait for user to press ON */
    clear_screen();
    while (keyb_isON()) ;	/* wait for user to release ON */
}

int main (int argc, char **argv)
{
    char name[256];		/* name of an object in the dir */
    SatAddr dirObj;		/* object in the directory */
    SatAddr taggedObj;		/* the tagged object to put on stack */
    SatAddr entry;		/* directory entry */
    int i = 0;

    // The following line is only really necessary for HPGCC 1.1
    unsigned rpl_stack_bias = sat_stack_init();

    clear_screen();

    // Here is the code to walk through the "entries" in a directory.
    // you use other methods to extract the name and object from an
    // entry. getCWD() gets the current directory.
    for (entry = DIRfirstEntry(getCWD());
	 entry;
	 entry = DIRnextEntry(entry)) {

	// Decode the entry to get the name and the object
	if (!DIRdecodeEntry(entry, name, &dirObj)) {
	    continue;		/* something went wrong */
	}

	printf("%s is %d nibbles\n", name, obNibbles(dirObj));

	// Print 5 entries at a time
	if (++i > 5) {
	    i = 0;
	    pause();
	}
    }
    pause();			/* pause so user can see last screen */

    // Clean up the stack (only necessary for HPGCC 1.1)
    sat_stack_exit(rpl_stack_bias);
    return 0;
}
