#include <hpgcc49.h>
#include "hpobjects.h"

//
// ACPTR routines
//

// Prolog (5 nibbles)
// Object's address (5 nibbles)
// Program's address (5 nibbles)
//++
bool isACPTR(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    return (prolog == SAT_DOACPTR);
}


// Extract the object and program addresses from an ACPTR object
bool ACPTRdecode(SatAddr src, SatAddr *obj, SatAddr *program)
{
    if (!isACPTR(src)) return false;
    if (obj) {
	*obj = sat_peek(src+5, 5);
    }
    if (program) {
	*program = sat_peek(src+10, 5);
    }
    return true;
}

// Put the library and command numbers into the xlib.  This
// assumes that dst points to an xlib object.
SatAddr ACPTRencode(SatAddr obj, SatAddr program, SatAddr dst)
{
    if (obj<0 || obj>0xfffff) return 0;
    if (program<0 || program>0xfffff) return 0;

    if (dst == 0) {
	dst = sat_createtemp(5+5+5);
	if (dst == 0) return 0;
    }
    sat_poke(dst, SAT_DOACPTR, 5);
    sat_poke(dst+5, obj, 5);
    sat_poke(dst+10, program, 5);
    return dst;
}

// Create an empty ACPTR. Contents are undefined.
SatAddr makeACPTR()
{
    return ACPTRencode(0,0,0);
}

