#include <hpgcc49.h>
#include "hpobjects.h"

/////////////////////////////////////////////////////////////////////
// BACKUP object
//++

// Is the object a BACKUP object
bool isBAK(SatAddr src)
{
    return (obProlog(src) == SAT_DOLIB);
}


// Return the number of characters in the BACKUP's name, or -1 on error
int BAKnameLen(SatAddr src)
{
    int result;
    if (!isBAK(src)) return -1;
    result = sat_peek(src+10,2);
    return result;
}


// Decode the BACKUP object at src.  If "name" is non-null then it is
// populated with the null-terminated name of the backup. "Name must
// have enough space to hold the name and terminating NULL, which will
// be 256 bytes maximum If "obj" is non-null then it will be populated
// with the address of the object in the backup.  If "crc" is non-null
// then it will be populated with the address of the CRC, which is
// BINT object.
bool BAKdecode(SatAddr src, char *name, SatAddr *obj, SatAddr *crc)
{
    int nameLen;
    if (!isBAK(src)) return false;

    nameLen = BAKnameLen(src);
    if (name) {
	sat_peek_sat_bytes(name, src+12, nameLen);
	name[nameLen] = 0;
    }
    if (obj) {
	*obj = src + 12 + 2*nameLen;
    }
    if (crc) {
	// Easiest way to find the CRC is to go backward from the end
      *crc = src + BAKnibbles(src) - 10;
    }
    return true;
}


// Size of the BACKUP object in nibbles
int BAKnibbles(SatAddr src)
{
    int result;
    if (!isBAK(src)) return -1;
    result = 5 + sat_peek(src+5, 5);
    return result;
}

