#include <hpgcc49.h>
#include "hpobjects.h"

// CMP (complex number) routines
//++
bool isCMP(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    bool result = (prolog == SAT_DOCMP);
    return result;
}


/* Convert an HP CMP object to C doubles. Returns true of false. */
bool CMPdecode(SatAddr src, double *re, double *im)
{
    if (!isCMP(src)) return false;
    if (re) {
	*re = sat_real2double(src+5);
    }
    if (im) {
	*im = sat_real2double(src+21);
    }
    return true;
}


// Convert C doubles re,im to an HP CMP object at Saturn address dst.
// returns true on success otherwise false.
SatAddr CMPencode(double re, double im, SatAddr dst)
{
    if (dst == 0) {
	dst = sat_createtemp(5+16+16);
    }
    sat_poke(dst, SAT_DOCMP, 5);
    double2hp(re, dst+5);
    double2hp(im, dst+5+16);
    return dst;
}

/* Create an uninitialized HP CMP object in temp space and returns
 * its address.
 */
SatAddr makeCMP()
{
    SatAddr result = sat_createtemp(5+16+16);
    sat_poke(result, SAT_DOCMP, 5);
    sat_memset(result+5, 0, 16+16);
    return result;
}

