#include <hpgcc49.h>
#include "hpobjects.h"

//
// CODE routines
//

// A Code object is encoded almost the same as a STRING, except that the
// length field is in nibbles, not bytes.
//++
bool isCODE(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    return (prolog == SAT_DOCODE);
}

// Given a CODE object at src, copy the code itself to dst, which must
// be big enough to accept it.  For example:
// CODE2m2m(src, malloc((CODEbytes(src))) will allocate
// space to hold the code and copy it there.
bool CODEdecode(SatAddr src, char *dst, int *nibbles)
{
    if (!isCODE(src)) return false;
    if (dst) {
	int bytes = CODEbytes(src);
	sat_peek_sat_bytes(dst, src+10, bytes);
    }
    *nibbles = CODEnibbles(src)-10;
    return true;
}


// Construct a CODE object at DST and copy "nibbles" nibbles there
// from "src".  Dst must point to enough space for the object.
SatAddr CODEencode(const char *src, const int nibbles, SatAddr dst)
{
    if (dst == 0) {
	dst = makeCODE(nibbles);
    }
    sat_poke(dst, SAT_DOCODE, 5);	  /* prolog */
    sat_poke(dst+5, nibbles+5, 5);	  /* size. This one is empty */
    sat_poke_nibbles(dst+10, src, nibbles);
    return dst;
}


/* Allocate space for a CODE object with nibbles nibbles of code, and
   put an empty CODE object there. Return the address of the CODE.*/
SatAddr makeCODE(unsigned nibbles)
{
    SatAddr result = sat_createtemp(nibbles+10);
    sat_poke(result, SAT_DOCODE, 5);	  /* prolog */
    sat_poke(result+5, nibbles+5, 5);	  /* size. */
    return result;
}

// Return the size in nibbles of the CODE object at src.
int CODEnibbles(SatAddr src)
{
    if (!isCODE(src)) return -1;
    return sat_peek(src+5, 5) + 5;
}


// Return the number of bytes necessary to hold the code in the CODE object
int CODEbytes(SatAddr src)
{
    // The code occupies CODEnibbles()-2 nibbles.  You need
    // (nibbles+1)/2 bytes (the +1 accounts for padding in case
    // nibbles is odd).
    if (!isCODE(src)) return -1;
    return (CODEnibbles(src) - 1) / 2;
}




