#include <hpgcc49.h>
#include "hpobjects.h"

// ECMP (complex number) routines
//
// The extended complex number uses extended reals for the re and im parts
//++
bool isECMP(SatAddr src)
{
    return (obProlog(src) == SAT_DOECMP);
}


/* Convert an HP ECMP object to C doubles. Returns true of false. */
bool ECMPdecode(SatAddr src, double *re, double *im)
{
    if (!isECMP(src)) return false;
    *re = ereal2double(src+5);
    *im = ereal2double(src+5+21);
    return true;
}


// Convert C doubles re,im to an HP ECMP object at Saturn address dst.
// returns true on success otherwise false.
SatAddr ECMPencode(double re, double im, SatAddr dst)
{
    if (dst == 0) {
	dst = sat_createtemp(5+21+21);
    }
    sat_poke(dst, SAT_DOECMP, 5);
    double2hpereal(re, dst+5);
    double2hpereal(im, dst+5+21);
    return dst;
}

/* Create an uninitialized HP ECMP object in temp space and returns
 * its address.
 */
SatAddr makeECMP()
{
    return ECMPencode(0.0, 0.0, 0);
}

int ECMPnibbles(SatAddr src)
{
    if (!isECMP(src)) return -1;
    return 5 + 21 + 21;
}
