#include <hpgcc49.h>
#include "hpobjects.h"


/////////////////////////////////////////////////////////////////////
//               FONT routines.
/////////////////////////////////////////////////////////////////////

//++
bool isFONT(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    return (prolog == SAT_DOFONT);
}

int FONTnibbles(SatAddr src)
{
    return 5 + sat_peek(src+5, 5);
}


//Return a pointer to the 16-nibble minifont entry corresponding to
//char "idx"
SatAddr FONTitem(SatAddr src, int idx)
{
    int offset;
    if (!isFONT(src)) return 0;
    if (idx<0 || idx>255) return 0;
    offset = 5+5+2+2+2*FONTgetNameLen(src)+2;
    return src+offset+16*idx;
}


// Set/get the minifont ID number
int FONTgetId(SatAddr src)
{
    int result;
    if (!isFONT(src)) return -1;
    result = sat_peek(src+5+5+2, 2);
    return result;
}

bool FONTsetId(SatAddr src, int id)
{
    if (!isFONT(src)) return false;
    if (id <0 || id>255) return false;
    sat_poke(src+5+5+2, id, 2);
    return true;
}


// Return the number of characters in the FONT name.
int FONTgetNameLen(SatAddr src)
{
    int result;
    if (!isFONT(src)) return -1;
    result = sat_peek(src+5+5+2+2, 2);
    return result;
}


// Decode a FONT into it's parts.  "src" points to the FONT, If
// non-null, ID will be set to FONT's ID number.  If "name" is
// non-null, then the font name will be copied to it as a
// null-terminated string.  You can get the number of characters in
// the name with FONTgetNameLen() ahead of time. If non-null, "height"
// will be set to the font height in pixels.  If "graphics" is
// non-null, then the font graphics will be copied there.  Graphics
// must be at least 2k bytes to hold the graphics.
bool FONTdecode(SatAddr src, int *id, char *name, int *height, char *graphics)
{
    if (!isFONT(src)) return false;
    if (id) {
	*id = FONTgetId(src);
    }
    if (name) {
	int chars = FONTgetNameLen(src);
	sat_peek_sat_bytes(name, src+5+5+2+2+2, chars);
	name[chars]=0;
    }
    if (height) {
	*height = sat_peek(src+5+5, 2);
    }
    if (graphics) {
	sat_peek_sat_bytes(graphics,
			   src+5+5+2+2+2*FONTgetNameLen(src)+2,
			   256*2*8);
    }
    return true;
}


// Change the name of the font at "src" to the name given by the
// null-terminated string in "name".  If "src" is not a font or if
// src's font name length is not the length of "name", then return false.
bool FONTsetName(SatAddr src, const char *name)
{
    int newNameLen;

    if (!isFONT(src)) return false;
    newNameLen = strlen(name);
    if (newNameLen != FONTgetNameLen(src)) return false;

    sat_poke_nibbles(src+5+5+2+2+2, name, newNameLen*2);
    return true;
}


// This is the oppositte of FONTdecode.  It will populate the FONT
// object at "dst". If dst is 0, then a new FONT object will be
// created.  Returns the address of FONT
SatAddr FONTencode(int id,
		   const char *name,
		   int height,
		   const char *graphics,
		   SatAddr dst)
{
    int nameLen = strlen(name);
    if (id<0 || id>0xff) return 0;
    if (height<0 || height>0xff) return 0;

    if (dst == 0) {
	dst = sat_createtemp(18+2*nameLen+4096);
    }
    sat_poke(dst, SAT_DOFONT, 5);			 /* prolog */
    sat_poke(dst+5, 5+5+2+2+2+2*nameLen+2+4096, 5);	 /* size */
    sat_poke(dst+10, height, 2);			 /* font height */
    sat_poke(dst+12, id, 2);				 /* id */
    sat_poke(dst+14, nameLen, 2);			 /* name len */
    sat_poke_nibbles(dst+16, name, nameLen*2);		 /* name */
    sat_poke(dst+16+2*nameLen, nameLen, 2); 		 /* second copy of namelen */
    sat_poke_nibbles(dst+18+2*nameLen, graphics, 4096);  /* graphics */
    return dst;
}


// Make a blank font with room for "nameLen" characters in the name
SatAddr makeFONT(int nameLen)
{
    SatAddr dst = sat_createtemp(18+2*nameLen+4096);

    // Populate some of the parts
    sat_poke(dst, SAT_DOFONT, 5);		/* prolog */
    sat_poke(dst+5, 13+2*nameLen+4096, 5);	/* size */
    sat_poke(dst+10, 8, 2);			/* font height */
    sat_poke(dst+14, nameLen, 2);		/* name len */
    sat_poke(dst+16+2*nameLen, nameLen, 2);	/* second copy of namelen */
    return dst;
}

