#include <hpgcc49.h>
#include "hpobjects.h"

// Library data is free-formed so there isn't much to do with it

//++

// Is the "src" address a LIBDATA
bool isLIBDATA(SatAddr src)
{
    return (obProlog(src) == SAT_DOLIBDATA);
}
// Return the size in nibbles of the LIBDATA object at src.
int LIBDATAnibbles(SatAddr src)
{
    if (!isLIBDATA(src)) return -1;
    return 5 + sat_peek(src+5, 5);
}


// Return the number of bytes required to store the data in a LIBDATA
// object
int LIBDATAbytes(SatAddr src)
{
    if (!isLIBDATA(src)) return -1;
    return (6 + sat_peek(src+5, 5)) / 2;
}


// Extract a LIBDATA's data and its size. If "data" is non-null then
// the data portion of the LIBDATA will be copied there, in which case
// it must contain enough space.  Note that LIBDATAbytes will tell you
// how much space is needed. If "nibbles" is non-null then the
// size in nibbles of the data is put there. Returns false if this
// isn't a LIBDATA object
bool LIBDATAdecode(SatAddr src, char *data, int *nibbles)
{
    if (!isLIBDATA(src)) return false;
    if (data) {
	sat_peek_sat_bytes(data, src+10, LIBDATAbytes(src));
    }
    if (nibbles) {
	*nibbles =  LIBDATAnibbles(src) - 10;
    }
    return true;
}


// Create a LIBDATA object with "nibbles" data starting at "data".
// Put the object in "dst". If dst is 0, then create a new
// LIBDATA. Returns the LIBDATA
SatAddr LIBDATAencode(const char *data, int nibbles, SatAddr dst)
{
    if (dst == 0) {
	dst = sat_createtemp(10+nibbles);
    }
    sat_poke(dst, SAT_DOLIBDATA, 5);
    sat_poke(dst+5, nibbles+5, 5);
    sat_poke_nibbles(dst+10, data, nibbles);
    return dst;
}


// Allocate space for a LIBDATA object with "nibbles" data in it.
//   The contents of the data will be undefined
SatAddr makeLIBDATA(unsigned nibbles)
{
    SatAddr dst = sat_createtemp(10+nibbles);
    sat_poke(dst, SAT_DOLIBDATA, 5);
    sat_poke(dst+5, nibbles+5, 5);
    return dst;
}

