#include <hpgcc49.h>
#include "hpobjects.h"

//
// Routines that deal with HP LIST object
//

//++

bool isLIST(SatAddr src)
{
    return (obProlog(src) == SAT_DOLIST);
}

// Allocate space for a LIST with NIBBLES data in it, and put an empty
// LIST at the beginning of the space. Return the address of the
// LIST. You can add data to the list with LISTadd(), as long as the
// total size of the items added doesn't exceed NIBBLES.
SatAddr makeLIST(unsigned nibbles)
{
    // Total space is space for the objects plus 5 nibbles for the
    // prolog and 5 nibbles for the composite
    SatAddr result = sat_createtemp(nibbles+10);
    sat_poke(result, SAT_DOLIST, 5);	  /* prolog */
    sat_poke(result+5, SAT_SEMI, 5);
    return result;
}



// Returns size of the LIST in nibbles.
int LISTnibbles(SatAddr src)
{
    if (!isLIST(src)) return -1;
    return compositeObNibbles(src);
}

// Returns the number of elements in the list.
int LISTsize(SatAddr src)
{
    if (!isLIST(src)) return -1;
    return compositeSize(src);
}

/* Make a copy of obj and put it in the list. This assumes that
   there is space in the list for the object. */
bool LISTadd(SatAddr list, SatAddr obj)
{
    if (!isLIST(list)) return false;
    return compositeAdd(list, obj);
}

 /* Concatenate an object to a list. This creates a new list
    consisting of the old one and the new object. Returns the address
    of the new list */
SatAddr LISTcat(SatAddr src, SatAddr obj)
{
    if (!isLIST(src)) return 0;
    return compositeCat(src, obj);
}

/* Merge two lists into one.  This creates a new list consisting of
   all the items in LIST1 followed by all the items in LIST2. For
   example, if list1 is {1 2 3} and list2 is {4 5 6} then
   LISTmerge(list1, list2) creates {1 2 3 4 5 6} */
SatAddr LISTmerge(SatAddr LIST1, SatAddr LIST2)
{
    if (!(isLIST(LIST1) && isLIST(LIST2))) return 0;
    return compositeMerge(LIST1, LIST2);
}

// Return the first object in a list, or 0 of the list is nil
SatAddr LISTfirstEntry(SatAddr list)
{
    if (!isLIST(list)) return 0;
    return compositeFirstEntry(list);
}


// Given an entry from a list returned by LISTfirstEntry or
// LISTnextEntry, return the next entry in the list.
SatAddr LISTnextEntry(SatAddr entry)
{
    return compositeNextEntry(entry);
}

// Return a pointer to the first object in the list, or zero if "list"
// isn't a list or if the list is empty.  "Iter" is the address of a
// scratch pointer used by LISTfirstOb/LISTnextOb to keep track of the
// position in the list. Do not modify iter between calls.  This
// dereferences indirect pointers in the list the ensure that it
// returns a pointer to an object
SatAddr LISTfirstOb(SatAddr list, SatAddr *iter)
{
    if (!isLIST(list)) return 0;
    return compositeFirstOb(list, iter);
}


// Given an iter that was previously passed to LISTfirstOb or
// LISTnextOb, return the address of the next object in the list and
// update iter.  This dereferences indirect pointers in the list the
// ensure that it returns a pointer to an object
SatAddr LISTnextOb(SatAddr *iter)
{
    return compositeNextOb(iter);
}



// Make a new list consisting of the objects ob1, ob2, ... obn and put
// them in dst. If dst is zero the space is allocated for the new
// list. Otherwise dst must contain sufficient space. Return the new list
SatAddr LISTencode(SatAddr dst, ... /* ob1, ob2, ob3, ..., obn , 0 */ )
{
    va_list argp;

    va_start(argp,dst);
    dst = compositeEncode(dst, argp);
    sat_poke(dst, SAT_DOLIST, 5);
    va_end(argp);
    return dst;
}

// Make a new list consisting of the objects obs[0], obs[1], obs[2],
// ... obs[n-1] and put them in dst. If dst is zero then space is
// allocated for the new list. Otherwise dst must contain sufficient
// space. Return the new list. Basically this is like ->LIST on the
// calculator
SatAddr LISTencodeN(int n, SatAddr obs[], SatAddr dst)
{
    dst = compositeEncodeN(n,obs,dst);
    if (dst) sat_poke(dst, SAT_DOLIST, 5);
    return dst;    
}


