#include <hpgcc49.h>
#include "hpobjects.h"

////////////////////////////////////////////////////////////
//              Stack routines.
////////////////////////////////////////////////////////////

//++

// The hplib has sat_pushptr() but not sat_popptr().  This pops one
// address off the stack and returns it. Returns 0 if stack is empty.
SatAddr STACKpop()
{
    SatAddr sp = sat_peek(SAT_DSKTOP, 5);
    SatAddr result = sat_peek(sp, 5);
    if (result) {
	sat_poke(SAT_DSKTOP, sp+5, 5);
    }
    return result;
}

// C version of calculator's DEPTH command.
int STACKdepth()
{
    // See section 61, page 140 of "Programming in Saturn Assembly
    // Language" for an explanation.  I have verified this on a 50g
    // with Nosy.  The stack is right before the command line.  It
    // contains 5 NULL nibbles to signify the end, and the pointers
    // themselves.
#define EDITLINE 0x806fd
    SatAddr sp = sat_peek(SAT_DSKTOP, 5);
    SatAddr bottom = sat_peek(EDITLINE, 5);
    int result = (bottom-sp)/5-1;
    return result;
}

/// \brief pick an object from an arbitrary level off the stack
SatAddr STACKpick(int level)
{
    if (level < 1 || level > STACKdepth()) return 0;
    SatAddr sp = sat_peek(SAT_DSKTOP, 5);
    SatAddr result = sat_peek(sp + 5 * (level-1), 5);
    return result;
}

