#include <hpgcc49.h>
#include "hpobjects.h"

/////////////////////////////////////////////////////////////////////
//         Symbolic expression.

// These functions have identical meeting to the equivalent ones
// for COL objects (RPL programs, shown above).  In other words,
// SYMBxyz has the same symantics as COLxyz.  See the comments for
// the COL functions for their meaning

//++

// Is the "src" address a SYMB object
bool isSYMB(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    return (prolog == SAT_DOSYMB);
}

extern  bool COLSYMBdecode(SatAddr src, char *dst, int *nibbles);

bool SYMBdecode(SatAddr src, char *dst, int *nibbles)
{
    if (!isSYMB(src)) return false;
    return COLSYMBdecode(src,dst, nibbles);
}

SatAddr SYMBencode(const char *src, const int nibbles, SatAddr dst)
{
    if ((dst = COLencode(src, nibbles, dst))) {
	sat_poke(dst, SAT_DOSYMB, 5); /* fix the prolog */
    }
    return dst;
}

SatAddr makeSYMB(unsigned nibbles)
{
    SatAddr result = sat_createtemp(nibbles+10);
    sat_poke(result, SAT_DOSYMB, 5);	  /* prolog */
    sat_poke(result+5, SAT_SEMI, 5);	  /* size. This one is empty */
    return result;
}

int SYMBnibbles(SatAddr src)
{
    if (!isSYMB(src)) return -1;
    return compositeObNibbles(src);
}

int SYMBbytes(SatAddr src)
{
    if (!isSYMB(src)) return -1;
    return (SYMBnibbles(src)-9) / 2;
}

// Return the first object in a symb, or 0 of the symb is nil
SatAddr SYMBfirstEntry(SatAddr symb)
{
    if (!isSYMB(symb)) return 0;
    return compositeFirstEntry(symb);
}


// Given an entry from a symb returned by SYMBfirstEntry or
// SYMBnextEntry, return the next entry in the symb.
SatAddr SYMBnextEntry(SatAddr entry)
{
    return compositeNextEntry(entry);
}

// Return a pointer to the first object in the symb, or zero if "symb"
// isn't a symb or if the symb is empty.  "Iter" is the address of a
// scratch pointer used by SYMBfirstOb/SYMBnextOb to keep track of the
// position in the symb. Do not modify iter between calls.  This
// dereferences indirect pointers in the symb the ensure that it
// returns a pointer to an object
SatAddr SYMBfirstOb(SatAddr symb, SatAddr *iter)
{
    if (!isSYMB(symb)) return 0;
    return compositeFirstOb(symb, iter);
}


// Given an iter that was previously passed to SYMBfirstOb or
// SYMBnextOb, return the address of the next object in the symb and
// update iter.  This dereferences indirect pointers in the symb the
// ensure that it returns a pointer to an object
SatAddr SYMBnextOb(SatAddr *iter)
{
    return compositeNextOb(iter);
}

SatAddr SYMBencodeV(SatAddr dst, ... /* ob1, ob2, ob3, ..., obn , 0 */ )
{
    va_list argp;

    va_start(argp,dst);
    dst = compositeEncode(dst, argp);
    sat_poke(dst, SAT_DOSYMB, 5);
    va_end(argp);
    return dst;
}

SatAddr SYMBencodeN(int n, SatAddr obs[], SatAddr dst)
{
    dst = compositeEncodeN(n,obs,dst);
    if (dst) sat_poke(dst, SAT_DOSYMB, 5);
    return dst;    
}

