#include <hpgcc49.h>
#include "hpobjects.h"

/////////////////////////////////////////////////////////////////////
//         Tagged object
// prolog (5 nibbles)
// tag size (2 nibbles). This is the number of characters in tag
// tag char 1 (2 nibbles)
// tag char 2 (2 nibbles)
// ...
// tag char n
// object (variable length)

// Note that the object can be a direct object or an indirect pointer.
//++

bool isTAG(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    return (prolog == SAT_DOTAG);
}

// Return the number of tag characters in teh TAG pointed at by src.
int TAGnumChars(SatAddr src)
{
    int result = sat_peek(src+5, 2);
    return result;
}


// Make a tagged object from the tag and object.  Tag is a
// null-terminated string.  obj is an object.  This makes a new
// objects consisting of the tag and a copy of obj.
SatAddr TAGencode(const char *tag, SatAddr obj, SatAddr dst)
{
    int i;
    int tagLen = strlen(tag);
    int nibbles = 7;		/* prolog and tag size */
    int objSize = objOrPtrNibbles(obj);

    if (objSize < 0) return 0;
    nibbles += 2*tagLen;
    nibbles += objSize;

    if (dst==0) {
	dst = makeTAG(nibbles);
    }
    sat_poke(dst, SAT_DOTAG, 5);
    sat_poke(dst+5, tagLen, 2);
    for(i=0; i < tagLen; ++i) {
	sat_poke(dst+7+2*i, tag[i], 2);
    }
    copyObjOrPtr(obj, dst + 7 + 2*tagLen);
    return dst;
}




// Decode a TAG object.  If "tag" is non-null, then this copies the
// tag into it and adds a null terminator.  If obj is non-null, then this
// points it at the object within the TAG.
bool TAGdecode(SatAddr src, char *tag, SatAddr *obj)
{
    int tagLen;
    if (!isTAG(src)) return false;
    tagLen = TAGnumChars(src);
    if (tag) {
	sat_peek_sat_bytes(tag, src+7, tagLen);
	tag[tagLen] = 0;
    }
    if (obj) {
	*obj = src + 7 + 2*tagLen;
    }
    return true;
}


SatAddr makeTAG(int nibbles)
{
    SatAddr result = sat_createtemp(nibbles);
    sat_poke(result, SAT_DOTAG, 5);
    return result;
}

// Return the size of the tag objects in nibbles
int TAGnibbles(SatAddr src)
{
    SatAddr obj, end;
    int tagLen;

    tagLen = TAGnumChars(src);
    obj = src+7+2*tagLen;
    end = skipOb(obj);
    if (end <= 0) return -1;
    return end - src;
}
