#include <hpgcc49.h>
#include "hpobjects.h"

/////////////////////////////////////////////////////////////////////
//         Symbolic expression.

// These functions have identical meeting to the equivalent ones
// for COL objects (RPL programs, shown above).  In other words,
// UNITxyz has the same symantics as COLxyz.  See the comments for
// the COL functions for their meaning

extern bool COLSYMBdecode(SatAddr src, char *dst, int *nibbles);
extern bool COLSYMBnibbles(SatAddr src);
extern bool COLSYMBbytes(SatAddr src);

//++

// Is the "src" address a UNIT object
bool isUNIT(SatAddr src)
{
    int prolog = sat_peek_sat_addr(src);
    return (prolog == SAT_DOUNIT);
}

// Given a UNIT object at src, copy the code itself (everything between
// the prolog and SEMI to dst. If dst is null, then a new UNIT will be
// created. Otherwise dst must be big enough to accept it; Note that
// this does NOT copy the prolog or composite themselves.
bool UNITdecode(SatAddr src, char *dst, int *nibbles)
{
    if (!isUNIT(src)) return false;
    return COLSYMBdecode(src, dst, nibbles);
}

// Construct a UNIT object at DST and copy "nibbles" nibbles there
// from "src".  The src should not include the prolog or terminating SEMI,
// which will be added automatically. If dst is 0 then space will be
// allcated for the object. OTherwise it must be big enough to hold it
// (nibbles+10)
SatAddr UNITencode(const char *src, const int nibbles, SatAddr dst)
{
    SatAddr result = COLencode(src, nibbles, dst);
    if (result == 0) return 0;
    sat_poke(result, SAT_DOUNIT, 5);
    return result;
}

SatAddr makeUNIT(unsigned nibbles)
{
    SatAddr result = makeCOL(nibbles);
    sat_poke(result, SAT_DOUNIT, 5); /* fix the prolog */
    return result;
}

int UNITnibbles(SatAddr src)
{
    if (!isUNIT(src)) return -1;
    return compositeObNibbles(src);
}

int UNITbytes(SatAddr src)
{
    if (!isUNIT(src)) return -1;
    return COLSYMBbytes(src);
}


// Return the first object in a unit, or 0 of the unit is nil
SatAddr UNITfirstEntry(SatAddr unit)
{
    if (!isUNIT(unit)) return 0;
    return compositeFirstEntry(unit);
}


// Given an entry from a unit returned by UNITfirstEntry or
// UNITnextEntry, return the next entry in the unit.
SatAddr UNITnextEntry(SatAddr entry)
{
    return compositeNextEntry(entry);
}

// Return a pointer to the first object in the unit, or zero if "unit"
// isn't a unit or if the unit is empty.  "Iter" is the address of a
// scratch pointer used by UNITfirstOb/UNITnextOb to keep track of the
// position in the unit. Do not modify iter between calls.  This
// dereferences indirect pointers in the unit the ensure that it
// returns a pointer to an object
SatAddr UNITfirstOb(SatAddr unit, SatAddr *iter)
{
    if (!isUNIT(unit)) return 0;
    return compositeFirstOb(unit, iter);
}


// Given an iter that was previously passed to UNITfirstOb or
// UNITnextOb, return the address of the next object in the unit and
// update iter.  This dereferences indirect pointers in the unit the
// ensure that it returns a pointer to an object
SatAddr UNITnextOb(SatAddr *iter)
{
    return compositeNextOb(iter);
}



// Make a new unit consisting of the objects ob1, ob2, ... obn and put
// them in dst. If dst is zero the space is allocated for the new
// unit. Otherwise dst must contain sufficient space. Return the new unit
SatAddr UNITencodeV(SatAddr dst, ... /* ob1, ob2, ob3, ..., obn , 0 */ )
{
    va_list argp;

    va_start(argp,dst);
    dst = compositeEncode(dst, argp);
    sat_poke(dst, SAT_DOUNIT, 5);
    va_end(argp);
    return dst;
}

// Make a new unit consisting of the objects obs[0], obs[1], obs[2],
// ... obs[n-1] and put them in dst. If dst is zero then space is
// allocated for the new unit. Otherwise dst must contain sufficient
// space. Return the new unit. Basically this is like ->UNIT on the
// calculator
SatAddr UNITencodeN(int n, SatAddr obs[], SatAddr dst)
{
    dst = compositeEncodeN(n,obs,dst);
    if (dst) sat_poke(dst, SAT_DOUNIT, 5);
    return dst;    
}
